@@ -127,7 +127,7 @@ model = Transfusion(
         dim = 64,
         depth = 4,
         dim_head = 32,
-        heads = 8
+        heads = 8,
     )
 ).cuda()
 
